import os


class Properties(object):

    def __init__(self, path):
        self.__props = {}
        full_path = os.path.abspath(path)

        if not os.path.isfile(full_path):
            full_path = os.path.abspath(path)

        self.__read_file(full_path)

    def __read_file(self, full_path):
        with open(full_path) as f:
            for line in f:
                line = line.strip()
                if not line or line.startswith('#'):
                    continue
                key, value = line.split('=')
                self.__props[key] = value

    def key(self, key, default=None):
        if key in self.__props:
            return self.__props[key]
        return default

    def key_int(self, key, default=None):
        value = self.key(key, default)
        if value:
            return int(value)
        return default

    def key_bool(self, key, default=None):
        value = self.key(key, default)
        if value:
            return value == "True"
        return default



import threading
import properties
from collections import namedtuple
from functools import wraps
from psycopg2.pool import ThreadedConnectionPool


_pool = None
_local = threading.local()

DatabasePropertiesClass = namedtuple('DatabaseProperties', 'name user password host show_sql')


def __read_db_properties(filepath):
    props = properties.Properties(filepath)

    DBNAME = props.key('db.name')
    USER = props.key('db.user')
    PASSWORD = props.key('db.password')
    HOST = props.key('db.host')
    SHOW_SQL = props.key_bool('db.show_sql')

    return DatabasePropertiesClass(name=DBNAME, user=USER, password=PASSWORD, host=HOST, show_sql=SHOW_SQL)


def start_db(dbproperties_filepath):
    return __read_db_properties(dbproperties_filepath)


def create_pool(database):
    global _pool
    _pool = ThreadedConnectionPool(1, 20, dbname=database.name, user=database.user,
                                   host=database.host, password=database.password)
    _pool.show_sql = database.show_sql


def _requires_local():
    if not hasattr(_local, 'connection'):
        _local.connection = None
    return _local


class TransactionalControl(object):
    def __enter__(self):
        pass

    def __exit__(self, error, value, traceback):
        connection = _requires_local().connection
        if connection:
            if error:
                connection.rollback()
            else:
                connection.commit()

        if error:
            raise value


def transaction(fnc):
    @wraps(fnc)
    def inner(*args, **kwargs):
        with TransactionalControl():
            fnc(*args, **kwargs)
    return inner


def my_connection():
    local = _requires_local()
    if not local.connection:
        local.connection = DatabaseConnection()
    return local.connection


def close():
    if not _local.connection:
        _local.connection.close()
        _local.connection = None


class DatabaseConnection(object):

    def __init__(self):
        self.cur = None
        self.connection = None

    def execute(self, query, values=None):
        cur = self.__cursor()
        self.__log_sql(cur, query, values)
        if values:
            cur.execute(query, values)
        else:
            cur.execute(query)
        return self

    def close(self):
        if self.cur:
            self.commit()
            if not self.cur.closed:
                self.cur.close()

    def commit(self):
        if self.connection:
            self.connection.commit()
            _pool.putconn(self.connection)
            self.connection = None

    def rollback(self):
        if self.connection:
            self.connection.rollback()
            if not self.cur.closed:
                self.cur.close()
            _pool.putconn(self.connection)
            self.connection = None

    def fetchall(self):
        if self.cur:
            return self.cur.fetchall()
        raise ConnException("No cursor to fetch")

    def description(self):
        if self.cur:
            columns = [column.name for column in self.cur.description]
            return columns
        return []

    @staticmethod
    def __log_sql(cur, query, values):
        global _pool
        if _pool.show_sql:
            print(cur.mogrify(query, values))

    def __cursor(self):
        global _pool
        if not self.connection:
            self.connection = _pool.getconn()

        if self.cur:
            if not self.cur.closed:
                self.cur.close()
        self.cur = self.connection.cursor()

        return self.cur


class ConnException(Exception):
    pass




def frozen():
    def set_attr(self, name, value):
        raise AttributeError("You cannot add attributes to %s" % self)
    return set_attr


class Enum(object):
    __setattr__ = frozen()

    class __metaclass__(type):
        __setattr__ = frozen()

    @classmethod
    def to_value(cls, key):
        if hasattr(cls, key):
            return getattr(cls, key)
        raise Exception("%s doesnt have the key: %s" % (cls.__name__, key))

    @classmethod
    def keys_values(cls):
        keys_and_values = {}

        for prop in cls.__dict__:
            if not prop.startswith('_') and \
               not isinstance(cls.__dict__[prop], staticmethod):
                keys_and_values[str(prop)] = cls.__dict__[prop]

        return keys_and_values

    @classmethod
    def values(cls):
        all_values = []

        for i in cls.__dict__:
            if not i.startswith('_') and \
               not isinstance(cls.__dict__[i], staticmethod):
                all_values.append(cls.__dict__[i])

        return all_values

    @classmethod
    def exist(cls, property):
        all_properties = cls.keys_values()
        return property in all_properties



from test_connection_db import TestLoader
from mock import Age, Person
import storage
import connection
from storage import Query


class TestDatabaseAPIUsage(TestLoader):

    def test_insert(self):
        self.assertTrue(storage.insert(Age('mouse', 20)))
        test_obj = storage.find(Age)
        self.assertIsNotNone(test_obj)

    def test_select(self):
        for i in range(0, 2):
            self.assertTrue(storage.insert(Age('mouse', 20)))

        test_obj = storage.find(Age)
        self.assertEqual(len(test_obj), 2)

    def test_select_one(self):
        for i in range(20, 22):
            name = 'mouse %s' % (i)
            self.assertTrue(storage.insert(Age(name, i)))

        test_obj = storage.find(Age, limit=1,
                                where={'age': 20, 'name': 'mouse 20'})

        self.assertEqual(1, len(test_obj))
        self.assertTrue(isinstance(test_obj[0], Age))
        self.assertEqual(test_obj[0].name, 'mouse 20')
        self.assertEqual(test_obj[0].age, 20)

    def test_select_limit(self):
        for i in range(20, 23):
            self.assertTrue(storage.insert(Age('mouse', i)))

        result = storage.find(Age, limit=1)
        self.assertEqual(1, len(result))

    def test_select_in(self):
        for i in range(20, 23):
            self.assertTrue(storage.insert(Age('mouse', i)))

        result = storage.find(Age, where={'age': (Query.IN, [20, 21, 23])})
        self.assertEqual(2, len(result))

    def test_delete(self):
        storage.insert(Age('mouse', 20))
        storage.delete(Age)

        saved_objects = storage.find(Age)
        self.assertEqual(len(saved_objects), 0)

        model = storage.put(Person(name='alabama'))
        storage.put(Person(name='orm'))
        storage.delete(Person, uuid=model.uuid)

        saved_objects = storage.find(Person)
        self.assertEqual(len(saved_objects), 1)

    def test_update_all(self):
        for i in range(10, 50, 10):
            self.assertTrue(storage.insert(Age('mouse', i)))

        self.assertTrue(storage.update(Age('mouse', 50)))
        test_obj = storage.find(Age, limit=10, where={'age': 50, 'name': 'mouse'})

        self.assertEqual(4, len(test_obj))
        self.assertTrue(isinstance(test_obj[0], Age))

        self.assertEqual(test_obj[0].name, 'mouse')
        self.assertEqual(test_obj[0].age, 50)

    def test_update(self):
        for i in range(10, 50, 10):
            self.assertTrue(storage.insert(Age('mouse', i)))

        self.assertTrue(storage.update(Age('mouseUpdated', 50),
                        where={'age': 10, 'name': 'mouse'}))

        test_obj = storage.find(Age, limit=10,
                                where={'age': 50, 'name': 'mouseUpdated'})

        self.assertEqual(1, len(test_obj))
        self.assertTrue(isinstance(test_obj[0], Age))
        self.assertEqual(test_obj[0].name, 'mouseUpdated')
        self.assertEqual(test_obj[0].age, 50)

    def test_query(self):
        storage.insert(Age('mouse', 22))
        storage.insert(Person('mouse', 'mickey', 23, 22.2))
        storage.insert(Person('mouse', 'minnie', 23, 22.2))

        q = Query(Person, alias='p').select(['p.age', 'a.age', 'p.lastname']) \
                                    .join(Age, alias='a', on='a.name = p.name') \
                                    .where({'p.lastname': 'mickey', 'a.age': 22})

        r = ('SELECT p.age,a.age,p.lastname FROM tb_Person p '
             'JOIN tb_Age a ON a.name = p.name WHERE 1=1 AND '
             'p.lastname=%s AND a.age=%s LIMIT 1000', ['mickey', 22])
        self.assertEqual(r, q.query())

        objects = storage.query(q)
        self.assertEqual(len(objects), 2)
        age, person = objects['Age'][0], objects['Person'][0]

        self.assertEqual(person.age, 23)
        self.assertEqual(person.lastname, 'mickey')
        self.assertEqual(age.age, 22)

        q = Query(Person, alias='p')\
            .select(['p.*'])\
            .join(Age, alias='a', on='a.name = p.name')\
            .where({'p.lastname': 'mickey', 'a.age': 22})

        r = ('SELECT p.* FROM tb_Person p JOIN tb_Age a ON a.name = '
             'p.name WHERE 1=1 AND p.lastname=%s AND '
             'a.age=%s LIMIT 1000', ['mickey', 22])
        self.assertEqual(r, q.query())

        objects = storage.query(q)
        self.assertEqual(len(objects), 1)
        person = objects['Person'][0]

        self.assertEqual(person.age, 23)
        self.assertEqual(person.lastname, 'mickey')

    def test_query_in(self):
        storage.insert(Age('mouse', 22))
        storage.insert(Person('mouse', 'mickey', 23, 22.2))
        storage.insert(Person('mouse', 'minnie', 23, 22.2))

        q = Query(Person).select(["*"])\
                         .where({'lastname': ('mickey', 'minnie')}, Query.IN)
        a = storage.query(q)
        self.assertEqual(len(a[Person.__name__]), 2)

    def test_put(self):
        p = Person('philip', 'folk', 20)
        self.assertIsNone(p.uuid)
        p = storage.put(p)
        self.assertIsNotNone(p.uuid)

        p.name = 'petter'
        new_p = storage.put(p)
        self.assertEqual(new_p.uuid, p.uuid)

    def test_query_get_all(self):
        uuids = []
        uuids.append(storage.put(Person('mouse1', 'minnie1', 23, 22.2)).uuid)
        uuids.append(storage.put(Person('mouse2', 'minnie2', 23, 22.2)).uuid)
        uuids.append(storage.put(Person('mouse3', 'minnie3', 23, 22.2)).uuid)
        uuids.append(storage.put(Person('mouse4', 'minnie4', 23, 22.2)).uuid)
        persons = storage.get_all(Person, uuids)
        self.assertEquals(len(persons), 4)

    def test_join(self):
        storage.put(Age(name='alabama'))
        storage.put(Person(name='alabama', lastname='felipe'))
        storage.put(Person(name='alabama', lastname='alabama'))

        query = Query(Person, alias='p').select(['p.*'])\
                                        .join(Age, alias='a', on='p.name = a.name')\
                                        .where({'p.name': 'alabama'})\
                                        .order("p.lastname DESC")

        results = storage.query(query)
        self.assertEqual(1, len(results))
        self.assertEqual('felipe', results['Person'][0].lastname)
        self.assertTrue('Person' in results)
        self.assertIsNotNone(results['Person'])

    def test_find(self):
        pass

    def test_get(self):
        person = storage.put(Person(name='alabama'))

        person_from_db = storage.get(Person, uuid=person.uuid)
        self.assertEqual(person.name, person_from_db.name)
        self.assertEqual(person.uuid, person_from_db.uuid)


class TestStorageCreateDatabase(TestLoader):

    @connection.transaction
    def create_db_just_person(self):
        storage.create_database(Person)

    @connection.transaction
    def create_db(self):
        storage.create_database([Person, Age])

    @connection.transaction
    def drop_person(self):
        sql = 'DROP TABLE tb_Person;'
        connection.my_connection().execute(sql)

    @connection.transaction
    def drop_all(self):
        sql = 'DROP TABLE tb_Person; DROP TABLE tb_Age;'
        connection.my_connection().execute(sql)

    def test_create_database(self):
        self.drop_person()
        self.create_db_just_person()
        self.assertEqual(0, len(storage.find(Person)))

        self.drop_all()
        self.create_db()
        self.assertEqual(0, len(storage.find(Person)))
        self.assertEqual(0, len(storage.find(Age)))



from collections import OrderedDict


class BadFormedQuery(Exception):
    pass


class Query(object):
    EQUAL = "="
    NOT_EQUAL = "!="
    IN = " IN "
    NOT_IN = " NOT IN "
    BIGGER_THAN = " > "
    LOWER_THAN = " < "
    LIKE = " LIKE "
    OR = " OR "
    AND = " AND "

    def __init__(self, cls, alias=None):
        self.base_table = cls.table_name()
        self.selects = None
        self.join_query = ""
        self.wheres = None
        self.query_str = None
        self.select_text = None
        self.limit_select = 1000
        self.params_where = []
        self.params_values = []
        self.param_custom_where = None
        self.order_by = None
        self.operator_where = None

        if not alias:
            self.tables_used = {'default': cls}
            return

        self.tables_used = {alias: cls}
        self.base_table = self.base_table + ' ' + alias

    def select(self, selects):
        self.select_text = ''
        if not isinstance(selects, list):
            selects = [selects]
        self.selects = selects
        for i in selects:
            self.select_text += i + ','
        self.select_text = self.select_text[:-1]
        return self

    def join(self, instance, alias=None, on=None):
        if not alias or not on:
            raise BadFormedQuery('Alias and on need be informed')

        if 'default' in self.tables_used:
            raise BadFormedQuery('Main table must have alias')

        self.tables_used[alias] = instance
        self.join_query += ' JOIN ' + instance.table_name()
        self.join_query += ' ' + alias + ' ON ' + on
        return self

    def split_select(self, columns=None):
        cls_fields = OrderedDict()

        if len(self.selects) < 2:

            sel = self.selects[0]
            if '*' in sel:

                alias = 'default'
                if '.' in sel:
                    alias = sel.split('.')[0]

                cls = self.tables_used[alias]
                cls_fields[cls] = []

                for i, column in enumerate(columns):
                    cls_fields[cls].append((column, i))

                return cls_fields

        count = 0

        for sel in self.selects:
            alias, field_name = sel.split('.')
            cls = self.tables_used[alias]

            if cls in cls_fields:
                cls_fields[cls].append((field_name, count))
            else:
                cls_fields[cls] = [(field_name, count)]

            count += 1

        return cls_fields

    def where(self, params_where, operator=EQUAL, operator_where=AND):
        self.operator_where = operator_where
        self.params_where.append((params_where, operator))
        return self

    def values(self, params_values):
        self.params_values = params_values
        return self

    def split_values(self):
        if not self.params_values:
            return None
        return self.params_values.values()

    def split_where(self):
        values = []
        for params, _ in self.params_where:
            values.extend(params.values())
        return values

    def limit(self, lim):
        if lim:
            self.limit_select = lim
        return self

    def delete(self):
        query = 'DELETE FROM %s' % self.base_table

        if len(self.params_where) > 0:
            query += self.__where()
        return query, self.split_where()

    def order(self, order_by):
        self.order_by = order_by
        return self

    def update(self):
        query = 'UPDATE %s' % self.base_table

        if self.params_values:
            values_str = Query.__join_values(self.params_values, "=", ',')
            values_str = values_str[1:]
            query += ' SET ' + values_str

        if len(self.params_where) > 0:
            query += self.__where()

        values = self.split_values()
        where = self.split_where()
        if where:
            values.extend(where)
        return query, values

    def insert(self):
        query = 'INSERT INTO %s (' % self.base_table
        query += self.__info_separator(self.params_values, ',') + ')'
        query += ' VALUES ('
        values = []
        for _ in self.params_values:
            values.append('%s')
        query += self.__info_separator(values, ',') + ')'
        return query, self.split_values()

    def query(self):
        if self.select_text:
            self.query_str = 'SELECT ' + \
                             self.select_text + ' FROM ' + self.base_table
        if self.join_query:
            self.query_str += self.join_query

        if len(self.params_where) > 0:
            self.query_str += self.__where()

        if self.order_by:
            self.query_str += ' ORDER BY ' + self.order_by

        if self.select_text:
            self.query_str += ' LIMIT ' + str(self.limit_select)

        return self.query_str, self.split_where()

    def __where(self):
        query = ""
        for params, operator in self.params_where:
            query += self.__join_values(params, operator,
                                        self.operator_where)

        if any(query):
            pos = len(self.operator_where)
            query = query[pos:]
            query = Query.AND + query
        return ' WHERE 1=1' + query

    @staticmethod
    def __info_separator(values, separator):
        ret = ''
        for i in values:
            ret += i + separator
        ret = ret[:-1]
        return ret

    @staticmethod
    def __igual_separator(values, separator):
        value = ''
        for i in values:
            value += separator + i + '= %s'
        return value

    @staticmethod
    def __join_values(values, operator, separator):
        value = ''
        for key in values:
            value += separator + key + operator + '%s'
        return value



import unittest
from storage import Query
from mock import Age, Person


class TestQuerys(unittest.TestCase):

    def test_insert_query(self):
        sql = Query(Age).values({'name': 'mouse', 'age': 22.5}).insert()
        self.assertEqual("INSERT INTO " + Age.table_name() + " (age,name) VALUES (%s,%s)", sql[0])
        self.assertEqual([22.5, 'mouse'], sql[1])

    def test_select_query(self):
        result = ("SELECT * FROM " + Age.table_name() + " WHERE 1=1 AND name=%s LIMIT 30", ['mickey'])
        query = Query(Age).select(['*']).where({"name": 'mickey'}).limit(30).query()
        self.assertEqual(result, query)

        result = ("SELECT * FROM " + Age.table_name() + "\
 WHERE 1=1 AND age=%s AND name=%s LIMIT 30", [20, 'mickey'])
        query = Query(Age).select(['*']).where({"age": 20, "name": 'mickey'}).limit(30).query()
        self.assertEqual(result, query)

        result = ("SELECT * FROM " + Age.table_name() + " \
WHERE 1=1 AND age=%s OR name=%s LIMIT 30", [20, 'mickey'])
        query = Query(Age).select(['*']).where({"age": 20, "name": 'mickey'}, operator_where=Query.OR).limit(30).query()
        self.assertEqual(result, query)

        result = ("SELECT * FROM " + Age.table_name() + " WHERE 1=1 AND name=%s LIMIT 1", ['mickey'])
        query = Query(Age).select(['*']).where({"name": 'mickey'}).limit(1).query()
        self.assertEqual(result, query)

        result = ("SELECT * FROM " + Age.table_name() + " LIMIT 1", [])
        query = Query(Age).select(['*']).limit(1).query()
        self.assertEqual(result, query)

        result = ("SELECT a.name FROM " + Age.table_name() + " a WHERE 1=1 AND a.name=%s LIMIT 1", ['mickey'])
        query = Query(Age, "a").select(['a.name']).where({"a.name": 'mickey'}).limit(1).query()
        self.assertEqual(result, query)

    def test_select_join_query(self):
        q = Query(Person, alias='p').select(['a.age', 'p.lastname']) \
                                    .join(Age, alias='a', on='a.name = p.name') \
                                    .where({'p.lastname': 'mickey', 'a.age': 22})

        r = ('SELECT a.age,p.lastname FROM tb_Person p JOIN tb_Age a \
ON a.name = p.name WHERE 1=1 AND p.lastname=%s AND a.age=%s LIMIT 1000', ['mickey', 22])
        self.assertEqual(r, q.query())

    def test_select_where_compost(self):
        q = Query(Person, alias='p').select(['a.age', 'p.lastname']) \
                                    .where({'p.lastname': 'mickey', 'a.age': 22}) \
                                    .where({'p.lastname': 'mouse'}, Query.NOT_EQUAL)

        r = ('SELECT a.age,p.lastname FROM tb_Person p WHERE 1=1 AND p.lastname=%s \
AND a.age=%s AND p.lastname!=%s LIMIT 1000', ['mickey', 22, 'mouse'])
        self.assertEqual(r, q.query())

    def test_delete(self):
        q = Query(Age).delete()
        r = ("DELETE FROM " + Age.table_name(), [])
        self.assertEqual(q, r)

        q = Query(Age).where({'name': 'mouse'}).delete()
        r = ("DELETE FROM " + Age.table_name() + " WHERE 1=1 AND name=%s", ['mouse'])
        self.assertEqual(q, r)

        q = Query(Age).where({'name': 'mouse', 'age': 20}).delete()
        r = ("DELETE FROM " + Age.table_name() + " WHERE 1=1 AND age=%s AND name=%s", [20, 'mouse'])
        self.assertEqual(q, r)

    def test_update(self):
        q = Query(Age).values({"age": 22, "name": "minnie"}).where({"age": 20, "name": "mickey"}).update()
        r = ("UPDATE " + Age.table_name() + " SET age=%s,name=%s \
WHERE 1=1 AND age=%s AND name=%s", [22, "minnie", 20, "mickey"])
        self.assertEqual(q, r)

        q = Query(Age).values({"age": 22, "name": "minnie"}).update()
        r = ("UPDATE " + Age.table_name() + " SET age=%s,name=%s", [22, "minnie"])
        self.assertEqual(q, r)



import unittest
import connection
from storage import Query
from models import BaseModel, StringProperty, IntegerProperty, FloatProperty, EnumProperty, BooleanProperty
from enum import Enum


class Age(BaseModel):
    name = StringProperty()
    age = FloatProperty()

    def __init__(self, name=None, age=None):
        self.name = name
        self.age = age

    def name(self):
        pass


class Gender(Enum):
    male = "male"
    famale = 'female'


class Person(BaseModel):
    active = BooleanProperty()
    name = StringProperty()
    lastname = StringProperty(140)
    age = IntegerProperty()
    weight = FloatProperty()
    gender = EnumProperty(Gender)

    def __init__(self, name=None, lastname=None, age=10, weight=23):
        self.name = name
        self.lastname = lastname
        self.age = age
        self.weight = weight


class TestLoader(unittest.TestCase):

    def setUp(self):
        database = connection.start_db('tests/db.properties')
        connection.create_pool(database)
        sql = 'CREATE TABLE ' + Age.table_name() \
              + ' (name varchar, age int, uuid varchar);'

        sql += 'CREATE TABLE ' + Person.table_name() + \
               ' (name varchar, lastname varchar, \
               age int, weight int, uuid varchar)'
        execute(sql)

    def tearDown(self):
        sql = 'DROP TABLE ' + Age.table_name() + ';'
        sql += 'DROP TABLE ' + Person.table_name() + ';'
        execute(sql)


def clear():
    connection.my_connection().execute("SELECT t.table_name FROM "
                                       "information_schema.tables "
                                       "t where t.table_schema = 'public'")

    result = connection.my_connection().fetchall()
    sql = ""
    for i in result:
        sql += "TRUNCATE " + i[0] + " CASCADE;"

    connection.my_connection().execute(sql)


def reset():
    try:
        clear()
    finally:
        connection.my_connection().close()


def execute(sql):
    try:
        connection.my_connection().execute(sql)
    except Exception as e:
        print(e)
    finally:
        connection.my_connection().close()



import unittest
from mock import Person
from models import PostgreSQLTypes


class TestModelMethods(unittest.TestCase):

    def test_to_json(self):
        fake = Person(name='testa', age=10)
        self.assertEqual({'age': 10, 'lastname': None, 'name': 'testa', 'weight': 23}, fake.to_json())

    def test_to_instance(self):
        model_json = {'name': 'john', 'age': 53}
        instance = Person.to_instance(model_json)
        self.assertTrue(isinstance(instance, Person))
        self.assertEqual(instance.name, 'john')
        self.assertEqual(instance.age, 53)

    def test_create_table(self):
        types = [PostgreSQLTypes.char, PostgreSQLTypes.boolean,
                 PostgreSQLTypes.real, PostgreSQLTypes.int]
        sql = "CREATE TABLE tb_Person (uuid {0}(36) PRIMARY KEY, \
active {1}, age {3}, gender {0}, lastname {0}(140), name {0}(512), weight {2});".format(*types)
        self.assertEqual(sql, Person.create_table_sql())

    def test_columns(self):
        self.assertEqual(['active', 'age', 'gender', 'lastname', 'name', 'uuid', 'weight'], Person.columns())
        p = Person()
        self.assertEqual(['active', 'age', 'gender', 'lastname', 'name', 'uuid', 'weight'], p.columns())



import inspect
import uuid
from properties import Properties


class PostgreSQLTypes(object):
    char = 'CHAR'
    boolean = 'BOOLEAN'
    int = 'INT'
    bigint = 'BIGINT'
    real = 'real'


class BaseProperty(object):

    def __init__(self):
        self.property_name = None

    def __get__(self, obj, objtype=None):
        if not obj:
            return self

        if self.property_name not in obj.__dict__:
            obj.__dict__[self.property_name] = None

        return obj.__dict__[self.property_name]

    def __set__(self, obj, value):
        self._validate(value)
        obj.__dict__[self.property_name] = value

    def _validate(self, value):
        pass

    def _type(self):
        raise NotImplementedError

    def __repr__(self):
        return str(self.__class__)


class StringProperty(BaseProperty):

    def __init__(self, size=512):
        self._size = size

    def _validate(self, value):
        if value and not (isinstance(value, str)):
            raise ValueError("The value " + self.property_name +
                             " must be a string, but was: " + str(value))

    def _type(self):
        return '%s(%s)' % (PostgreSQLTypes.char, self._size)


class BooleanProperty(BaseProperty):

    def _validate(self, value):
        if value and not (isinstance(value, bool)):
            raise ValueError("The value must be bool")

    def _type(self):
        return PostgreSQLTypes.boolean


class EnumProperty(BaseProperty):

    def __init__(self, cls):
        self.enum_type = cls

    def _validate(self, value):
        if value and not self.enum_type.exist(value):
            raise ValueError("This EnumProperty:" + value +
                             " is not valid to EnumType:" +
                             self.enum_type.__name__)

    def _type(self):
        typeof = type(self.enum_type.values()[0]).__name__
        return {'str': PostgreSQLTypes.char, 'int': PostgreSQLTypes.int}[typeof]


class IntegerProperty(BaseProperty):
    def _validate(self, value):
        if value and not (isinstance(value, int) or isinstance(value, long)):
            raise ValueError("The value " +
                             self.property_name + " must be a integer")

    def _type(self):
        return PostgreSQLTypes.int


class BigIntegerProperty(IntegerProperty):

    def _type(self):
        return PostgreSQLTypes.bigint


class FloatProperty(BaseProperty):

    def _type(self):
        return PostgreSQLTypes.real


class ModelBasicProtocol(type):

    @staticmethod
    def _mount_property_details(class_dict):
        description = {}

        for attribute in class_dict:
            if isinstance(class_dict[attribute], BaseProperty):
                # set the name of the column to the column object
                class_dict[attribute].property_name = attribute
                description[attribute] = class_dict[attribute]

        return description

    def __new__(cls, name, parents, dct):
        description = {}
        # add to _properties_details all properties that the classes
        # that this Property inherity.
        for parent in parents:
            desc = ModelBasicProtocol._mount_property_details(parent.__dict__)
            description.update(desc)

        # add to _properties_details all properties that this class has
        description.update(ModelBasicProtocol._mount_property_details(dct))

        dct["_properties_details"] = description

        if '__init__' in dct:
            init_method = dct['__init__']
            init_inspection = inspect.getargspec(init_method)

            if init_inspection.defaults is None \
               and len(init_inspection.args) > 1:
                raise Exception('A modelobject must have '
                                'a keywords arguments in __init__')

        return type.__new__(cls, name, parents, dct)


def generate_id():
    return uuid.uuid4().hex


class BaseModel(object):
    __metaclass__ = ModelBasicProtocol

    uuid = StringProperty()

    def __init__(self, *args, **kargs):
        for k, value in kargs.items():
            if k in dir(self):
                setattr(self, k, value)

    def to_json(self):
        return_json = {}

        for key, _ in sorted(self._properties_details.items()):
            if key in self.__dict__:

                value = self.__dict__[key]
                if not isinstance(value, Properties):
                    return_json[key] = value

        return return_json

    @classmethod
    def to_instance(cls, json):
        return cls(**json)

    @classmethod
    def table_name(cls):
        return 'tb_%s' % (cls.__name__,)

    @classmethod
    def columns(cls):
        return [column_name for column_name, column_class in sorted(cls._properties_details.items())]

    @classmethod
    def create_table_sql(cls):
        uuid = "uuid %s(36) PRIMARY KEY," % (PostgreSQLTypes.char)
        fields = ""

        for prop, prop_type in sorted(cls._properties_details.items()):
            if prop == 'uuid':
                continue

            fields += "%s %s, " % (prop, prop_type._type())

        fields = fields[0:-2]

        return "CREATE TABLE %s (%s %s);" % (cls.table_name(), uuid, fields,)



from mock import Age, Person, TestLoader
import storage
from storage import Query


class TestSQLInjection(TestLoader):

    def test_sql_injection(self):
        self.assertTrue(storage.insert(Age('mouse', 20)))
        test = Age("'; delete from tb_Age")
        self.assertTrue(storage.insert(test))
        test_obj = storage.find(Age, where={'name': 'mouse'})
        self.assertEqual(len(test_obj), 1)


class TestModelDefaultArgs(TestLoader):

    def test_default(self):
        self.assertTrue(storage.insert(Person('mouse', 'optico', 20, 21)))
        q = Query(Person, alias='u').select(['u.name', 'u.age'])
        p = storage.query(q)['Person'][0]
        self.assertEqual(p.weight, None)
        self.assertEqual(p.lastname, None)



from query import Query
from models import generate_id
import connection


def insert(instance):
    values = instance.__dict__
    q, v = Query(instance.__class__).values(values).insert()
    connection.my_connection().execute(q, v)
    return True


def delete(cls, args=None, uuid=None):
    args = args or {}
    if uuid:
        args['uuid'] = uuid
    sql, values = Query(cls).where(args).delete()
    connection.my_connection().execute(sql, values)
    return True


def put(instance):
    if not instance.uuid:
        instance.uuid = generate_id()
        insert(instance)
        return instance

    if not exist(instance.__class__, uuid=instance.uuid):
        insert(instance)
        return instance

    update(instance, {'uuid': instance.uuid})
    return instance


def find(cls, limit=1000, where=None, order="", operator_where=Query.AND):
    where = where or {}
    q = Query(cls).select('*')
    deleted_keys = []

    for key in where:
        values = where[key]

        # example of usages:
        # .find(Person, where={'code_language': (Query.IN, ['ruby', 'python']))
        if isinstance(values, tuple):
            val = values[1]
            if isinstance(values[1], list):
                val = tuple(values[1])
            q = q.where({key: val}, values[0])
            deleted_keys.append(key)

    for key in deleted_keys:
        del where[key]

    q.where(where, operator_where=operator_where).limit(limit).order(order)
    result = query(q)
    if cls.__name__ in result:
        return result[cls.__name__]
    return []


def exist(cls, where=None, uuid=None):
    where = where or {}
    if uuid:
        where['uuid'] = uuid

    q = Query(cls).select(["count(*)"]).where(where).limit(1)
    sql, where = q.query()
    connection.my_connection().execute(sql, where)
    results = connection.my_connection().fetchall()
    return results[0][0] > 0


def get_all(cls, uuids=None, limit=30):
    uuids = uuids or []
    q = Query(cls).select('*')\
                  .where({'uuid': tuple(uuids)}, Query.IN)\
                  .limit(limit)
    r = query(q)
    if cls.__name__ in r:
        return r[cls.__name__]
    return []


def get(cls, where=None, uuid=None):
    where = where or {}
    if uuid:
        where['uuid'] = uuid

    rets = find(cls, 1, where)
    if len(rets):
        return rets[0]
    return None


def update(instance, where=None):
    where = where or {}
    values = instance.__dict__
    q, v = Query(instance.__class__).values(values).where(where).update()
    connection.my_connection().execute(q, v)
    return True


def query(query):
    sql, args = query.query()
    connection.my_connection().execute(sql, args)
    results = connection.my_connection().fetchall()
    class_with_fields = query.split_select(connection.my_connection().description())
    return __objects_resulted_from_join(results, class_with_fields)


def create_database(models):
    if not isinstance(models, list):
        models = [models]

    all_queries = ""
    for table in models:
        all_queries += table.create_table_sql()

    connection.my_connection().execute(all_queries)


def __objects_resulted_from_join(results, fields_of_class):
    ret = dict()
    for result in results:
        for cls in fields_of_class.keys():

            obj = cls()
            obj = __clean_properties(obj)

            for field_name in fields_of_class[cls]:
                value = result[field_name[1]]
                if value and isinstance(value, str):
                    value = value.strip()
                setattr(obj, field_name[0], value)

            if cls.__name__ in ret:
                ret[cls.__name__].append(obj)
            else:
                ret[cls.__name__] = [obj]
    return ret


def __clean_properties(instance):
    for prop in instance.__dict__:
        setattr(instance, str(prop), None)

    return instance



import unittest
from enum import Enum


class FakeEnum(Enum):
    an_prop = 'value'
    _private_prop = 'private_value'


class TestEnum(unittest.TestCase):

    def test_to_value(self):
        self.assertEqual(FakeEnum.to_value('an_prop'), 'value')
        with self.assertRaises(Exception):
            FakeEnum.to_value('wrong_prop')

    def test_keys_value(self):
        result = {'an_prop': 'value'}
        self.assertEqual(FakeEnum.keys_values(), result)

    def test_values(self):
        values = ['value']
        self.assertEqual(FakeEnum.values(), values)

    def test_exists(self):
        self.assertTrue(FakeEnum.exist('an_prop'))
        self.assertFalse(FakeEnum.exist('_private_prop'))



import unittest
from mock import Person, Gender
from models import BaseProperty


class TestProperty(unittest.TestCase):

    def test_enum_property(self):
        model = Person()
        model.gender = Gender.male
        self.assertEquals(model.gender, Gender.male)

        with self.assertRaises(ValueError):
            a = Person()
            a.gender = "invalid valid"

    def test_property(self):
        with self.assertRaises(ValueError):
            obj = Person()
            obj.name = 3

        with self.assertRaises(ValueError):
            obj = Person()
            obj.age = "a"

        obj = Person()
        obj.name = "string"
        obj.age = 1

        self.assertEqual(obj.name, "string")
        self.assertEqual(obj.age, 1)

    def test_different_object(self):
        obj1 = Person()
        obj1.name = "string"
        obj1.age = 1

        self.assertEqual(obj1.name, "string")
        self.assertEqual(obj1.age, 1)

        obj2 = Person()
        obj2.name = "new"
        obj2.age = 2

        self.assertEqual(obj2.name, "new")
        self.assertEqual(obj2.age, 2)

        self.assertEqual(obj1.name, "string")
        self.assertEqual(obj1.age, 1)

    def test_wrong_implementation_of_property(self):
        class WrongProp(BaseProperty):
            pass

        with self.assertRaises(NotImplementedError):
            WrongProp()._type()



import os
from setuptools import setup

setup(
    name="alabama",
    version="0.1.1",
    author="Felipe Volpone",
    author_email="felipevolpone@gmail.com",
    description=("A small and simple python orm to connect with postgresql database."),
    license = "MIT",
    keywords = "orm postgresql alabama",
    url = "http://github.com/felipevolpone/alabama_orm",
    packages=['alabama', 'tests'],
    install_requires=['psycopg2'],
    long_description="Check on github: http://github.com/felipevolpone/alabama_orm",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Topic :: Utilities",
    ],
)



import unittest
import sys
import os


ROOT_PATH = os.path.dirname(__file__).split(os.path.sep)[0]

# include alabama dir to path
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))+"/alabama/")

if __name__ == '__main__':
    tests = unittest.TestLoader().discover(ROOT_PATH, "*.py")
    result = unittest.TextTestRunner(verbosity=2, failfast=False).run(tests)

    if not result.wasSuccessful():
        sys.exit(1)



import unittest
from models import ModelBasicProtocol, BaseModel, StringProperty


class TestModelIsImplementingMeta(unittest.TestCase):

    def __get_wrong_class(self):
        class Fake(object):
            __metaclass__ = ModelBasicProtocol

            def __init__(self, a):
                pass

        return Fake

    def __get_right_class(self):
        class Fake(object):
            __metaclass__ = ModelBasicProtocol

            def __init__(self, name=None):
                pass

        return Fake

    def __get_other_right_class(self):
        class Fake(object):
            __metaclass__ = ModelBasicProtocol

            def __init__(self):
                pass

        return Fake

    def test_is_implementing(self):

        with self.assertRaises(Exception):
            self.__get_wrong_class()(1)

        self.assertIsNotNone(self.__get_right_class()())
        self.assertIsNotNone(self.__get_other_right_class()())

        self.assertEqual('Fake', self.__get_other_right_class().__name__)


class TestBehaviorOfModelBasicProtocol(unittest.TestCase):

    def test_properties_details_method(self):
        class Animal(BaseModel):
            specie = StringProperty()

        class Dog(Animal):
            name = StringProperty()

        rex = Dog()
        properties = rex._properties_details

        self.assertTrue('specie' in properties)
        self.assertTrue('name' in properties)
